import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * This class will read in a set of bug reports, parse the bug reports, and
 * print out the desired attributes into a single large XML file.
 * 
 * @author schilling
 * 
 */
public class XMLBugReader {
	/**
	 * This is the main method. It will read in the set of bug reports and
	 * combine them into a single large XML file which represents the desired
	 * fields from all bug reports.
	 * 
	 * @param argv
	 *            argv[0] represents the output file which is to be created.
	 *            argv[1]-argv[n] are the bug reports that are to be included in
	 *            the listing.
	 * @throws Exception
	 *             An exception will be thrown if there is a problem handling
	 *             the input file.
	 */
	public static void main(String argv[]) throws Exception {
		// Instantiate a class that will hold the BugSet.
		BugSet b = new BugSet();
		FileWriter fw = new FileWriter(argv[0]);

		// Read in each of the bug reports and process one by one.
		for (int index = 1; index < argv.length; index++) {
			String filenameToRead = argv[index];

			// Indicate which file is being processed.
			System.out.println("Processing: " + filenameToRead);

			// Open up a reader to read the given stream.
			File rawInputFile = new File(filenameToRead);
			InputStream inputStream = new FileInputStream(rawInputFile);
			Reader reader = new InputStreamReader(inputStream, "UTF-8");

			
			InputSource is1 = new InputSource(reader);
			is1.setEncoding("UTF-8");
			
			// Create a document builder factory which will process the XML file.
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setValidating(false);
			DocumentBuilder db = dbf.newDocumentBuilder();

			// Parse the document.
			Document doc = db.parse(is1);
			doc.getDocumentElement().normalize();
			
			// Get a list of the files that have the type bug associated with them.
			NodeList nodeLst = doc.getElementsByTagName("bug");
			
			// Step over the nodes, trying to find the ones we care about.
			for (int s = 0; s < nodeLst.getLength(); s++) {
				Node fstNode = nodeLst.item(s);

				if (fstNode.getNodeType() == Node.ELEMENT_NODE) {
					// Instantiate a new bug instance.
					Bug bug = new Bug(Integer.parseInt(getNodeText(fstNode,
							"bug_id")));
					bug.setShortDescription(getNodeText(fstNode, "short_desc"));
					bug.setBugStatus(getNodeText(fstNode, "bug_status"));
					bug.setResolution(getNodeText(fstNode, "resolution"));
					bug.setPriority(getNodeText(fstNode, "priority"));
					bug.setBugSeverity(getNodeText(fstNode, "bug_severity"));
					bug.setVersion(Double.parseDouble(getNodeText(fstNode,
							"version")));
					bug.setOs(getNodeText(fstNode, "op_sys"));
					b.data.add(bug);
				}
			}
			// Close the stream and go on to the next one.
			inputStream.close();
		}

		// Create an XML representation of the final product.
		JAXBContext context = JAXBContext.newInstance(BugSet.class);
		Marshaller m = context.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
		m.marshal(b, fw);
		fw.close();
	}

	/**
	 * This method will return the text from a given XML node as a string. Only
	 * the text will be returned.
	 * 
	 * @param n
	 *            n represents the node.
	 * @param field
	 *            This is the name of the field that is to be returned.
	 * @return The return value will be the ASCII text associated with this
	 *         node.
	 */
	private static String getNodeText(Node n, String field) {
		String retVal = "";

		Element sndElmnt = (Element) n;
		NodeList sndNmElmntLstTxt = sndElmnt.getElementsByTagName(field);

		Element sndNmElmnt = (Element) sndNmElmntLstTxt.item(0);
		if (sndNmElmnt == null) {// DO nothing

		} else {
			NodeList sndNm = sndNmElmnt.getChildNodes();
			if (sndNm.getLength() > 0) {
				retVal = ((Node) sndNm.item(0)).getNodeValue();
			}
		}
		return retVal;

	}
}